# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-03-17 17:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_competition', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='competition',
            name='voting_open',
            field=models.BooleanField(default=True, verbose_name='True if users can vote on the competition'),
        ),
        migrations.AlterField(
            model_name='competitionentry',
            name='description',
            field=models.TextField(help_text='You can use some markdown here.'),
        ),
    ]
