{% load i18n %}

<div class="django-competition-root vote-competition">

  <h3>{{ competition.name }}</h3>

  {{ competition.description_html }}

  <p><a>{% trans "You can vote at the bottom of the page." %}</a></p>

  <h3>{% trans "Competition entries" %}</h3>

  <p>{% trans "Entries are sorted randomly, for fairness." %}</p>

  {% for entry in entries %}
    <div class="django-competition-entry">
      <h4>{{ entry.name }}</h4>
      <div class="django-competition-entry-body">
        {{ entry.description_html }}
      </div>

    </div>
  {% endfor %}

  <h3 id="voting-anchor">{% trans "Vote now!" %}</h3>
  <form method="POST">
    {% csrf_token %}
    {{ form.as_p }}
    <button type="submit">Submit vote</button>
  </form>
</div>
