{% load i18n %}

<div class="django-competition-root vote-confirm">
  {% if form_error %}
    <h3>{% trans "You can't confirm this vote right now" %}</h3>

    <p>{% trans "Error is:" %} {{ form_error }}</p>

  {% else %}
    <h3>{% trans "Vote confirmation." %}</h3>

    <p>
      {% trans "You need to confirm your vote by clicking the button below." %}
    </p>

    <h4>{% trans "Vote details" %}</h4>

    <p>{% blocktrans with competition_name=vote.competition.name %}
      You are voting in competition {{ competition_name }}.
    {% endblocktrans %}</p>

    {{ competition.description_html }}

    <p>{% trans "You have selected following entries" %}</p>
    <ul>
    {% for entry in vote.entries %}
      <li>
        <p>{{ entry.name }}</p>
        {{ entry.description_html }}
      </li>
    {% endfor %}
    </ul>

    <p>{% trans "To confirm your vote please click button below." %}</p>
    <form method="POST">
      {% csrf_token %}
      <button type="submit" class="btn btn-primary">
        {% trans "I confirm this vote" %}
      </button>
    </form>
  {% endif %}
</div>
